{**********************************************************************
                PilotLogic Software House

 Package ds_Pas2JavaScript_IDE
 This file is part of CodeTyphon Studio (https://www.pilotlogic.com/)
***********************************************************************}

unit frmPas2jsSWServersManager;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, StdCtrls,
  ComCtrls, Buttons, ExtCtrls, ClipBrd,
  LazLoggerBase,
  IDECommands,
  IDEWindowIntf, IDEDialogs, IDEMsgIntf, LazIDEIntf,
  pjscontroller;

const
  cnPasJSSWSDlgWindowName = 'IDEPasJSSWSDlgWindow';  //===SOS SOS ct9999 ====

type

  { TPasJSSWSDlgWindow }

  TPasJSSWSDlgWindow = class(TForm)
    SWSMImages: TImageList;
    lbSWSCount: TLabel;
    lwForServers: TListView;
    Panel1: TPanel;
    Panel2: TPanel;
    ToolBar1: TToolBar;
    btUpdate: TToolButton;
    btDelete: TToolButton;
    btStart: TToolButton;
    ToolButton2: TToolButton;
    btClear: TToolButton;
    ToolButton3: TToolButton;
    btStop: TToolButton;
    btCopy: TToolButton;
    ToolButton6: TToolButton;
    procedure btCopyClick(Sender: TObject);
    procedure btDeleteClick(Sender: TObject);
    procedure btStartClick(Sender: TObject);
    procedure btStopClick(Sender: TObject);
    procedure btUpdateClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure lwForServersMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure btClearClick(Sender: TObject);
  private
  class var
    TheForm: TPasJSSWSDlgWindow;
    procedure DoControllerRefresh(Sender: TObject);
    procedure ServerInstanceToListItem(LI: TListItem; SI: TServerInstance);
    Procedure UpdateButtons;
  public
    constructor Create(AnOwner: TComponent); override;
    destructor Destroy; override;
    class function Instance: TPasJSSWSDlgWindow;
    procedure RefreshList;
  end;

var
  PasJSSWSDlgWindow: TPasJSSWSDlgWindow;
  PasJSSWSDlgWindowCreator: TIDEWindowCreator; // set by Register
  ViewPasJSSWSDlgWindowCommand: TIDECommand;   // set by Register

procedure ShowPasJSSWSDlgWindow(Sender: TObject);
procedure CreatePasJSSWSDlgWindow(Sender: TObject; aFormName: string; var AForm: TCustomForm; DoDisableAutoSizing: boolean);

implementation

uses strpas2jsdesign;

{$R *.frm}

//const
 // iiStopped = 0;
 // iiRunning = 1;

//=== ct9999 =======================================================
procedure ShowPasJSSWSDlgWindow(Sender: TObject);
begin
  IDEWindowCreators.ShowForm(cnPasJSSWSDlgWindowName, True);
end;

procedure CreatePasJSSWSDlgWindow(Sender: TObject; aFormName: string; var AForm: TCustomForm; DoDisableAutoSizing: boolean);
var
  xForm: TPasJSSWSDlgWindow;
begin
  if CompareText(aFormName, cnPasJSSWSDlgWindowName) <> 0 then
    exit;

  xForm := nil;
  IDEWindowCreators.CreateForm(xForm, TPasJSSWSDlgWindow, DoDisableAutoSizing, TyphonIDE.OwningComponent);

  xForm.Name := cnPasJSSWSDlgWindowName;
  AForm := xForm;
end;
//=====================================================================

constructor TPasJSSWSDlgWindow.Create(AnOwner: TComponent);
begin
  inherited Create(AnOwner);
  if PasJSSWSDlgWindow = nil then PasJSSWSDlgWindow := Self;  //=== SOS ====

  Caption := 'SWServers Manager';

  lbSWSCount.Caption := SafeFormat(SWebserversCount, ['0']);
  with lwForServers do
  begin
    Column[0].Caption := SWebserversPort;
    Column[1].Caption := SWebserversStatus;
    Column[2].Caption := SWebserversBaseDir;
    Column[3].Caption := SWebserversProject;
    Column[4].Caption := SWebserversExtra;
  end;
end;

destructor TPasJSSWSDlgWindow.Destroy;
begin
  inherited Destroy;
end;
//==============================================================================

procedure TPasJSSWSDlgWindow.FormShow(Sender: TObject);
begin
  TPJSController.Instance.OnRefresh := @DoControllerRefresh;
  RefreshList;
end;

procedure TPasJSSWSDlgWindow.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
  CloseAction := caFree;
end;

procedure TPasJSSWSDlgWindow.FormDestroy(Sender: TObject);
begin
  TPJSController.Instance.OnRefresh := nil;
  if (Self = TheForm) then
    TheForm := nil;
end;

Procedure TPasJSSWSDlgWindow.UpdateButtons;
Var B:Boolean;
begin
   B:=(lwForServers.Selected<>Nil);

   btDelete.Enabled:=B;
   btStart.Enabled:=B;
   btStop.Enabled:=B;
   btCopy.Enabled:=B;
end;

procedure TPasJSSWSDlgWindow.lwForServersMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  UpdateButtons;
end;

procedure TPasJSSWSDlgWindow.btClearClick(Sender: TObject);
var
  C: TPJSController;
begin
  C := TPJSController.Instance;
  if C = nil then exit;
  C.ServerInstances.Clear;

  RefreshList;
  UpdateButtons;
end;

procedure TPasJSSWSDlgWindow.btDeleteClick(Sender: TObject);
var
  SI: TServerInstance;
  C: TPJSController;
begin
  if lwForServers.Selected = nil then Exit;

  SI := TServerInstance(lwForServers.Selected.Data);
  if SI=Nil then Exit;
  SI.StopServer;

  C := TPJSController.Instance;
  if C = nil then exit;

  C.ServerInstances.Delete(SI.Index);
            
  RefreshList;
  UpdateButtons;
end;

procedure TPasJSSWSDlgWindow.btCopyClick(Sender: TObject);
var
 SL:TStringList;
begin
  if lwForServers.Selected = nil then Exit;

  try
  SL:=TStringList.Create;
  with lwForServers.Selected do
  begin
     SL.Assign(lwForServers.Selected.SubItems);
  end;
     Clipboard.AsText:=SL.Text;
  finally
    SL.Free;
  end;
end;

procedure TPasJSSWSDlgWindow.btStartClick(Sender: TObject);
var
  SI: TServerInstance; 
  i:Integer;
begin
  if lwForServers.Selected = nil then Exit;
                             
  i:=lwForServers.Selected.Index;
  SI := TServerInstance(lwForServers.Selected.Data);
  if SI=Nil then Exit;
  SI.StartServer;

  RefreshList;    
  lwForServers.Items[i].Selected:=True;
  UpdateButtons;
end;

procedure TPasJSSWSDlgWindow.btStopClick(Sender: TObject);
var
  SI: TServerInstance;
  i:Integer;
begin
  if lwForServers.Selected = nil then Exit;

  i:=lwForServers.Selected.Index;
  SI := TServerInstance(lwForServers.Selected.Data);
  if SI=Nil then Exit;
  SI.StopServer;

  RefreshList;
  lwForServers.Items[i].Selected:=True;
  UpdateButtons;
end;

procedure TPasJSSWSDlgWindow.btUpdateClick(Sender: TObject);
begin   
  RefreshList;  
  UpdateButtons;
end;

procedure TPasJSSWSDlgWindow.DoControllerRefresh(Sender: TObject);
begin
  RefreshList;  
  UpdateButtons;
end;

class function TPasJSSWSDlgWindow.Instance: TPasJSSWSDlgWindow;
begin
  if TheForm = nil then
    TheForm := TPasJSSWSDlgWindow.Create(Application);
  Result := TheForm;
end;

procedure TPasJSSWSDlgWindow.ServerInstanceToListItem(LI: TListItem; SI: TServerInstance);
var
  S, SError: string;
  im:Integer;
begin
  im:=-1;
  LI.Caption := IntToStr(SI.Port);
  if SI.Running then
  begin
    im := 1;
    S := SStatusRunning;
  end
  else if (SI.RunError <> '') then
  begin      
    im := 2;
    S := SStatusError;
    SError := SI.RunError;
  end else
  begin
    im := 3;
    S := SStatusStopped;
  end;

  LI.ImageIndex:=im;
  LI.SubItems.Add(S);
  LI.SubItems.Add(SI.BaseDir);
  LI.SubItems.Add(SI.LastProject);
  LI.SubItems.Add(SError);
  LI.Data := SI;
end;

procedure TPasJSSWSDlgWindow.RefreshList;
var
  C: TPJSController;
  I: integer;
  LI: TListItem;
  SI: TServerInstance;
begin
  C := TPJSController.Instance;
  if (C = nil) or (C.ServerInstances = nil) or (C.ServerInstances.Count = 0) then
  begin
    lwForServers.Items.Clear;
    lbSWSCount.Caption := SafeFormat(SWebserversCount, ['0']);
    exit;
  end;
  lbSWSCount.Caption := SafeFormat(SWebserversCount, [IntToStr(C.ServerInstances.Count)]);
  with lwForServers.Items do
    try
      BeginUpdate;
      Clear;
      for I := 0 to C.ServerInstances.Count - 1 do
      begin
        SI := C.ServerInstances[i];
        LI := Add;
        ServerInstanceToListItem(LI, SI);
      end;
    finally
      EndUpdate;
    end;
end;

end.
