{
  MIT License

  Copyright (c) 2018 Hélio S. Ribeiro and Anderson J. Gado da Silva

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
}

//=== ct9999 Modify for CodeTyphon Studio =======================

unit Pas2JS_IDE_Descriptor;

{$mode objfpc}{$H+}

interface

uses
  Classes,
  SysUtils,
  Forms,
  Controls,
  LazIDEIntf,
  ProjectIntf,
  CompOptsIntf,
  FormEditingIntf,
  PropEdits;

type

  { TPas2JSProject }

  TPas2JSProject = class(TProjectDescriptor)
  private
    FPas2JSBuilder: string;
    FPas2JSJSON: string;
  public
    constructor Create; override;
    function GetLocalizedName: string; override;
    function GetLocalizedDescription: string; override;
    function InitProject(AProject: TTyphonProject): TModalResult; override;
    function CreateStartFiles({%H-}AProject: TTyphonProject): TModalResult; override;
  end;

  { TPas2JSWForm }

  TPas2JSWForm = class(TFileDescPascalUnitWithResource)
  public
    constructor Create; override;
    function GetInterfaceUsesSection: string; override;
    function GetInterfaceSource(const {%H-}Filename, {%H-}SourceName, ResourceName: string): string; override;
    function GetLocalizedName: string; override;
    function GetLocalizedDescription: string; override;
  end;

  { TPas2JSWFrame }

  TPas2JSWFrame = class(TFileDescPascalUnitWithResource)
  public
    constructor Create; override;
    function GetInterfaceUsesSection: string; override;
    function GetInterfaceSource(const {%H-}Filename, {%H-}SourceName, ResourceName: string): string; override;
    function GetLocalizedName: string; override;
    function GetLocalizedDescription: string; override;
  end;

  { TPas2JSWDataModule }

  TPas2JSWDataModule = class(TFileDescPascalUnitWithResource)
  public
    constructor Create; override;
    function GetInterfaceUsesSection: string; override;
    function GetInterfaceSource(const {%H-}Filename, {%H-}SourceName, ResourceName: string): string; override;
    function GetLocalizedName: string; override;
    function GetLocalizedDescription: string; override;
  end;

//=== ct9999 =============================

var
  VPas2JSProject: TPas2JSProject;
  VPas2JSWForm: TPas2JSWForm;    
  VPas2JSWFrame: TPas2JSWFrame;   
  VPas2JSWDataModule: TPas2JSWDataModule;

implementation

uses
  TypInfo,
  FileUtil,
  WebCtrls,
  FPJSON,
  pjsprojectoptions, //=== ct9999 ==============
  ComponentEditors;

{ TPas2JSProject }

constructor TPas2JSProject.Create;
begin
  inherited Create;
  FPas2JSBuilder := '';
  FPas2JSJSON := '';
  Name := 'JS GUI Web Browser Application (Pas2JS)';    //=== ct9999 ===========
end;

function TPas2JSProject.GetLocalizedName: string;
begin
  Result := 'JS GUI Web Browser Application (Pas2JS)';  //=== ct9999 ===========
end;

function TPas2JSProject.GetLocalizedDescription: string;
begin
  Result := 'Create a Pas2JS Web GUI application';
end;

function TPas2JSProject.InitProject(AProject: TTyphonProject): TModalResult;

  function Project: TTyphonProjectFile;

    function Source: string;
    const
      LE = LineEnding;
    begin
      Result :=
        'program Project1; ' + LE +
        '' + LE +
        '{$mode delphi}{$H+}' + LE +
        '' + LE +
        'uses' + LE +
        '  Interfaces, Forms;' + LE +
        '' + LE +
        'begin' + LE +
        '  Application.Initialize;' + LE +
        '  Application.Run;' + LE +
        'end.';
    end;

  begin
    Result := AProject.CreateProjectFile('project1.ppr');  //=== ct9999 ===================
    Result.IsPartOfProject := True;
    Result.SetSourceText(Source, True);
    SetDefaultWebRunParams(AProject.RunParameters.GetOrCreate('Default'));
  end;

  function HTMLFile: TTyphonProjectFile;
  const
    TemplateHTMLSource =
         '<!doctype html>'+LineEnding
        +'<html lang="en">'+LineEnding
        +'<head>'+LineEnding
        +'  <meta http-equiv="Content-type" content="text/html; charset=utf-8">'+LineEnding
        +'  <meta name="viewport" content="width=device-width, initial-scale=1">'+LineEnding
        +'  <title>Project1</title>'+LineEnding
        +'  <script src="project1.js"></script>'+LineEnding
        +'</head>'+LineEnding
        +'<body>'+LineEnding
        +'  <script>'+LineEnding
        +'    rtl.run();'+LineEnding
        +'  </script>'+LineEnding
        +'</body>'+LineEnding
        +'</html>'+LineEnding;
  begin
    Result := AProject.CreateProjectFile('project1.html');
    Result.IsPartOfProject := True;
    AProject.CustomData.Values['PasJSHTMLFile'] := Result.Filename;
    AProject.CustomData['PasJSWebBrowserProject'] := '1';
    Result.CustomData['PasJSIsProjectHTMLFile'] := '1';
    Result.CustomData.Values['MaintainHTML'] := '1';
    Result.SetSourceText(TemplateHTMLSource);

  end;

var
  CompOpts: TTyphonCompilerOptions;
begin
  Result := inherited InitProject(AProject);
  AProject.AddFile(Project, False);
  AProject.AddFile(HTMLFile, True);
  //=== ct9999 =========================
  AProject.AddPackageDependency('js_BaseRTL');
  AProject.AddPackageDependency('js_WebWidgetsGUI');
  AProject.Flags := AProject.Flags - [pfRunnable];
  AProject.LoadDefaultIcon;
  AProject.MainFileID := 0;
  AProject.TyphonCompilerOptions.TargetFilename := 'project1';
  CompOpts := AProject.TyphonCompilerOptions;
  CompOpts.TargetOS:='Browser';
  CompOpts.GenerateDebugInfo := False;
  CompOpts.CompilerPath := '$MakeExe(IDE,$(FPCEXEDir)/pas2js)';
  CompOpts.CustomOptions := '-Jeutf-8 -Jirtl.js -Jc -Jminclude -JRjs';   
  CompOpts.CPPInline:=false;
  //=====================================
end;

function TPas2JSProject.CreateStartFiles(AProject: TTyphonProject): TModalResult;
begin
  TyphonIDE.DoOpenEditorFile('project1.html', -1, -1, [ofProjectLoading,ofRegularFile]);
  TyphonIDE.DoNewEditorFile(VPas2JSWForm, '', '', [nfIsPartOfProject, nfOpenInEditor, nfCreateDefaultSrc]);
  Result := mrOk;
end;

{ TPas2JSWForm }

constructor TPas2JSWForm.Create;
begin
  inherited Create;
  Name := 'WForm';
  ResourceClass := TWForm;
  UseCreateFormStatements := True;
end;

function TPas2JSWForm.GetInterfaceUsesSection: string;
begin
  Result := 'JS, Classes, SysUtils, Graphics, Controls, Forms, Dialogs, WebCtrls';
end;

function TPas2JSWForm.GetInterfaceSource(const Filename, SourceName, ResourceName: string): string;
const
  LE = LineEnding;
begin
  Result :=
    'type' + LE +
    '  T' + ResourceName + ' = class(' + ResourceClass.ClassName + ')' + LE +
    '  private' + LE + LE +
    '  public' + LE + LE +
    '  end;' + LE + LE;

  if (DeclareClassVariable) then
  begin
    Result := Result +
      'var' + LE +
      '  ' + ResourceName + ': T' + ResourceName + ';' + LE + LE;
  end;
end;

function TPas2JSWForm.GetLocalizedName: string;
begin
  Result := 'JS Web Form (Pas2JS)';  //=== ct9999 =========
end;

function TPas2JSWForm.GetLocalizedDescription: string;
begin
  Result := 'Create a Pas2JS Web Form, for JS GUI Web Browser App'; //=== ct9999 =======
end;

{ TPas2JSWFrame }

constructor TPas2JSWFrame.Create;
begin
  inherited Create;
  Name := 'WFrame';
  ResourceClass := TWFrame;
  UseCreateFormStatements := False;
end;

function TPas2JSWFrame.GetInterfaceUsesSection: string;
begin
  Result := 'JS, Classes, SysUtils, Graphics, Controls, Forms, Dialogs, WebCtrls';
end;
                       
function TPas2JSWFrame.GetInterfaceSource(const Filename, SourceName, ResourceName: string): string;
const
  LE = LineEnding;
begin
  Result :=
    'type' + LE +
    '  T' + ResourceName + ' = class(' + ResourceClass.ClassName + ')' + LE +
    '  private' + LE + LE +
    '  public' + LE + LE +
    '  end;' + LE + LE;
end;

function TPas2JSWFrame.GetLocalizedName: string;
begin
  Result := 'JS Web Frame (Pas2JS)';   //=== ct9999 ======
end;

function TPas2JSWFrame.GetLocalizedDescription: string;
begin
  Result := 'Create a Pas2JS Web Frame, for JS GUI Web Browser App'; //=== ct9999 =======
end;

{ TPas2JSWDataModule }

constructor TPas2JSWDataModule.Create;
begin
  inherited Create;
  Name := 'WDataModule';
  ResourceClass := TWDataModule;
  UseCreateFormStatements := True;
end;

function TPas2JSWDataModule.GetInterfaceUsesSection: string;
begin
  Result := 'JS, Classes, SysUtils, Graphics, Controls, Forms, Dialogs, WebCtrls';
end;

function TPas2JSWDataModule.GetInterfaceSource(const Filename, SourceName, ResourceName: string): string;
const
  LE = LineEnding;
begin
  Result :=
    'type' + LE +
    '  T' + ResourceName + ' = class(' + ResourceClass.ClassName + ')' + LE +
    '  private' + LE + LE +
    '  public' + LE + LE +
    '  end;' + LE + LE;

  if (DeclareClassVariable) then
  begin
    Result := Result +
      'var' + LE +
      '  ' + ResourceName + ': T' + ResourceName + ';' + LE + LE;
  end;
end;

function TPas2JSWDataModule.GetLocalizedName: string;
begin
  Result := 'JS Web Data Module (Pas2JS)';   //=== ct9999 =======================
end;

function TPas2JSWDataModule.GetLocalizedDescription: string;
begin
  Result := 'Create a Pas2JS Web Data Module, for JS GUI Web Browser App'; //=== ct9999 =======
end;

//=== ct9999 =============================


end.
